@@ -1,16 +1,13 @@
 import concurrent.futures
 import datetime
-import itertools
 import json
 import logging
 import os
 import time
-from collections import deque
-from dataclasses import asdict, astuple, dataclass
+from dataclasses import asdict, dataclass
 from decimal import Decimal
 
 from mysql.connector import MySQLConnection, connect
-from mysql.connector.pooling import MySQLConnectionPool
 
 
 @dataclass
@@ -110,6 +107,12 @@ def extract_keyvals(row: dict, keys: list[tuple[str, str]]):
     return new_dict
 
 
+def query_rows(con: MySQLConnection, query_statement: str, query_params) -> list[dict]:
+    with con.cursor(dictionary=True, buffered=True) as cur:
+        cur.execute(query_statement, tuple(query_params))
+        return cur.fetchall()
+
+
 class MysqlTableCompare:
     def __init__(
         self,
@@ -119,14 +122,11 @@ class MysqlTableCompare:
         src_table: str,
         dst_database: str,
         dst_table: str,
-        parallel: int = 1,
         limit_size: int = 2000,
     ) -> None:
         self.source_dsn = src_dsn
         self.target_dsn = dst_dsn
 
-        self.parallel = parallel
-
         self.limit_size = limit_size
 
         self.src_database = src_database
@@ -134,8 +134,6 @@ class MysqlTableCompare:
         self.dst_database = dst_database
         self.dst_table = dst_table
 
-        self.start_timestamp = time.time()
-
         self.compare_name: str = f"{self.src_database}.{self.src_table}"
         if self.src_database == self.dst_database and self.src_table != self.dst_table:
             self.compare_name += f".{self.dst_table}"
@@ -143,7 +141,6 @@ class MysqlTableCompare:
             self.compare_name += f".{self.dst_database}.{self.dst_table}"
 
         self.checkpoint_file = f"{self.compare_name}.ckpt.json"
-        self.done_file = f"{self.compare_name}.done"
         self.different_file = f"{self.compare_name}.diff.log"
 
     def write_different(self, rows: list[dict]):
@@ -170,7 +167,7 @@ class MysqlTableCompare:
                     ckpt.row[coln] = Decimal(ckpt.row[coln])
         return ckpt
 
-    def get_queery_full_table_statement_params(self, ckpt_row: dict = None):
+    def get_query_full_table_statement_params(self, database: str, table: str, ckpt_row: dict = None):
         _keyval = ckpt_row
         # select * from where 1 = 1 and ((a > xxx) or (a = xxx and b > yyy) or (a = xxx and b = yyy and c > zzz)) order by a,b,c limit checksize
         _key_colns = ", ".join([f"`{col[0]}`" for col in self.source_table_keys])
@@ -188,10 +185,10 @@ class MysqlTableCompare:
                 operator = ">" if i == end_idx else "="
                 condition_parts.append(f"`{column_name}` {operator} %s")
             where_conditions.append(" and ".join(condition_parts))
-        where_clause = "WHERE" + "(" + ") or (".join(where_conditions) + ")"
+        where_clause = "WHERE " + "(" + ") or (".join(where_conditions) + ")"
 
-        statement_with_condition = f"SELECT * FROM {self.src_database}.{self.src_table} {where_clause} ORDER BY {_key_colns} LIMIT {self.limit_size}"
-        statement_without_condition = f"SELECT * FROM {self.src_database}.{self.src_table} ORDER BY {_key_colns} LIMIT {self.limit_size}"
+        statement_with_condition = f"SELECT * FROM {database}.{table} {where_clause} ORDER BY {_key_colns} LIMIT {self.limit_size}"
+        statement_without_condition = f"SELECT * FROM {database}.{table} ORDER BY {_key_colns} LIMIT {self.limit_size}"
 
         _params: list = []
         if _keyval:
@@ -203,56 +200,53 @@ class MysqlTableCompare:
 
         return statement, _params
 
-    def query_rows(self, con: MySQLConnectionPool, query_statement: str, query_params) -> list[dict]:
-        with con.get_connection() as con:
-            con: MySQLConnection
-            with con.cursor(dictionary=True, buffered=True) as cur:
-                cur.execute(query_statement, tuple(query_params))
-                return cur.fetchall()
-
-    def compare_full_table(self):
+    def compare_full_table(self, source_con: MySQLConnection, target_con: MySQLConnection):
         _keyval = self.checkpoint.row
 
         while True:
-            query_statement, query_params = self.get_queery_full_table_statement_params(_keyval)
-            self.logger.info(f"query statment: '{query_statement}', params '{query_params}'")
+            self.logger.info(f"compare keys: {_keyval}")
+
+            query_statement_src, query_params_src = self.get_query_full_table_statement_params(self.src_database, self.src_table, _keyval)
+            query_statement_dst, query_params_dst = self.get_query_full_table_statement_params(self.dst_database, self.dst_table, _keyval)
+
+            # self.logger.info(f"query statment: '{query_statement_src}', params '{query_params}'")
+            # self.logger.info(f"query statment: '{query_statement}', params '{query_params}'")
 
             source_rows = []
             target_rows = []
+            diff_rows = []
 
             _start1 = time.time()
 
-            with concurrent.futures.ThreadPoolExecutor(max_workers=2) as executor:
-                future_source = executor.submit(self.query_rows, self.source_conpool, query_statement, query_params)
-                future_target = executor.submit(self.query_rows, self.target_conpool, query_statement, query_params)
+            for i in range(1, 3):
+                with concurrent.futures.ThreadPoolExecutor(max_workers=2) as executor:
+                    future_source = executor.submit(query_rows, source_con, query_statement_src, query_params_src)
+                    future_target = executor.submit(query_rows, target_con, query_statement_dst, query_params_dst)
 
-                source_rows = future_source.result()
-                target_rows = future_target.result()
+                    source_rows = future_source.result()
+                    target_rows = future_target.result()
 
-            self.logger.info(f"query source & target rows, source rows: {len(source_rows)}, target rows {len(target_rows)} {get_elapsed_time(_start1, 2)}s.")
+                diff_rows = find_missing_in_b(source_rows, target_rows)
+
+                if len(source_rows) == 0 or len(diff_rows) == 0:
+                    break
+
+                self.logger.info(f"diff count: {len(diff_rows)}, try {i}/2 recompare after sleep {5}s.")
+                time.sleep(5)
 
             if len(source_rows) == 0:
                 break
-            else:
-                _keyval = extract_keyvals(source_rows[-1], self.source_table_keys)
-                _start = time.time()
-                diff_rows = find_missing_in_b(source_rows, target_rows)
-                self.logger.info(f"find_missing_in_b {get_elapsed_time(_start, 2)}s.")
-                if len(diff_rows) >= 1:
-                    self.write_different(diff_rows)
-                self.checkpoint.row = _keyval
-                self.write_checkpoint()
-                self.logger.info(f"Missing elements in b: {len(diff_rows)}")
+
+            self.write_different(diff_rows)
+
+            _keyval = extract_keyvals(source_rows[-1], self.source_table_keys)
+            self.checkpoint.row = _keyval
+            self.write_checkpoint()
+            self.logger.info(f"query source & target rows, source rows: {len(source_rows)}, target rows {len(target_rows)} {get_elapsed_time(_start1, 2)}s.")
 
     def run(self) -> None:
         self.logger = init_logger(self.compare_name)
 
-        if os.path.exists(self.done_file):
-            self.logger.info(f"The comparison is complete and '{self.done_file}' exists.")
-            return
-
-        self.logger.info("init check.")
-
         with connect(**self.source_dsn) as source_con, connect(**self.target_dsn) as target_con:
             source_table_struct: list[tuple[str, str]] = get_table_structure(source_con, self.src_database, self.src_table)
             target_table_struct: list[tuple[str, str]] = get_table_structure(target_con, self.dst_database, self.dst_table)
@@ -273,24 +267,15 @@ class MysqlTableCompare:
             self.source_table_rows_number = max(1, get_table_rows_number(source_con, self.src_database, self.src_table))
             self.logger.info(f"source table rows number: {self.source_table_rows_number}.")
 
-        self.checkpoint = self.read_checkpoint()
-        self.processed_rows_number = self.checkpoint.processed
-        self.different_rows_number = self.checkpoint.different
+            self.checkpoint = self.read_checkpoint()
+            self.processed_rows_number = self.checkpoint.processed
+            self.different_rows_number = self.checkpoint.different
 
-        self.logger.info(f"from checkpoint: {self.checkpoint}")
+            self.logger.info(f"from checkpoint: {self.checkpoint}")
 
-        self.logger.info("source table connect pool create.")
-        self.source_conpool = MySQLConnectionPool(pool_name="source_conpool", pool_size=self.parallel, **self.source_dsn)
+            self.compare_full_table(source_con, target_con)  # main
 
-        self.logger.info("target table connect pool create.")
-        self.target_conpool = MySQLConnectionPool(pool_name="target_conpool", pool_size=self.parallel, **self.target_dsn)
-        self.compare_full_table()  # main
+            self.logger.info(f"compare completed, processed rows: {self.processed_rows_number}, different: {self.different_rows_number}.")
 
-        self.logger.info(
-            f"compare completed, processed rows: {self.processed_rows_number}, different: {self.different_rows_number},  elapsed time: {get_elapsed_time(self.start_timestamp, 2)}s."
-        )
-
-        open(self.done_file, "w").close()
-
-        if os.path.exists(self.checkpoint_file):
-            os.remove(self.checkpoint_file)
+            if os.path.exists(self.checkpoint_file):
+                os.remove(self.checkpoint_file)
